/* $Header: /src/carson/nistnet/RCS/nistnet.h,v 1.2 1997/10/20 17:11:39 carson Exp $ */

#ifndef _NISTNET_H
#define _NISTNET_H

/* nistnet.h - Linux implementation of "hitbox"-like functionality */

/* structure used for requesting the nistnet hitbox */
struct lin_hitreq {
	__u32	src;		/* source/destination to hit */
	__u32	dest;		/* either may be set to INADDR_ANY */
	unsigned int p_drop;	/* probability of drop times 65536 */
	unsigned int p_dup;	/* probability of duplicate times 65536 */
	unsigned int delay;	/* mean delay on packets in microseconds */
	unsigned int delsigma;	/* standard deviation on delay in usecs */
	unsigned int bandwidth;	/* max bandwidth (bytes/sec) on this pair */

	unsigned long qsize;	/* shouldn't be here ... */
	int	queue_mgr;	/* not used */
/* We actually implement the simpler DRD algorithm rather than RED,
 * but for our purposes they're close enough that these defines seem ok...
 */
	int	drd_max;	/* DRD max queue length */
#define red_th_max	drd_max
	int	drd_min;	/* DRD min queue length */
#define red_th_min	drd_min
};

#define MILLION	1000000		/* just so I don't do the wrong no. of 0's */
#define PROBFACTOR	65536	/* All probabilities are scaled by this */


/* structure used for returning stats on the hitbox */

struct lin_hitstats {
	struct lin_hitreq hitreq;
	int	n_drops;
	int	rand_drops;	/* random drops */
	int	mem_drops;	/* out of memory errors */
	int	drd_drops;	/* DRD algorithm drops */
#define	red_drops	drd_drops
	int	dups;
/* Statistics used in manipulating bandwidith */
	struct timeval next_packet; /* when next packet can be sent */
	unsigned long	current_size;	/* size of packet being sent currently */
	struct timeval last_packet; /* when last packet was sent */
	unsigned long	current_bandwidth;	/* Approximation of bandwidth used */
	unsigned long bytes_sent;
	unsigned long qlen;
#define BAND_ARRAY	10	/* bandwidth averaged over 10 seconds */
	unsigned long bandwidth_array[BAND_ARRAY];
	int seats_used;
/* None of these others are currently used */
	int	if_used;
	int	connection_count;
	int	gateway_mode;
	unsigned int mtu;
	int	red_save_arg;
	int	reset;
	int	virtual_timer;
};

struct lin_globalstats {	/* Global emulator statistics */
	unsigned long  process_overhead[BAND_ARRAY];
	unsigned long  unprocess_overhead[BAND_ARRAY];
	unsigned long  hash_tries[BAND_ARRAY];
};

struct addrpair {
	__u32	src;		/* source/destination to hit */
	__u32	dest;		/* either may be set to INADDR_ANY */
};

#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/icmp.h>
#include <linux/igmp.h>

struct lin_mungestats {
	struct iphdr ip;
	union {
		struct tcphdr tcp;
		struct udphdr udp;
		struct icmphdr icmp;
		struct igmphdr igmp;
	} h;
};
/* ioctl's */

/* hitbox */
#define HITIOCTL_OFF	_IO('h', 0)	/* master switch */
#define HITIOCTL_ON	_IO('h', 1)
#define HITIOCTL_ADD	_IOW('h', 2, struct lin_hitreq)
#define HITIOCTL_REMOVE	_IOW('h', 3, struct lin_hitreq)
#define HITIOCTL_STATS	_IOW('h', 4, struct lin_hitstats)
#define HITIOCTL_MODE	_IOW('h', 5, int)
#define HITIOCTL_TIMER	_IOW('h', 6, int)
#define HITIOCTL_MTU	_IOW('h', 7, int)
#define HITIOCTL_DEBUG	_IOW('h', 8, int)
#define HITIOCTL_GLOBALSTATS	_IOW('h', 9, struct lin_globalstats)

/* mungebox */
#define MUNGEIOCTL_OFF		_IO('m', 0)	/* master switch */
#define MUNGEIOCTL_ON		_IO('m', 1)
#define MUNGEIOCTL_WAIT		_IO('m', 2)	/* hold packets */
#define MUNGEIOCTL_NOWAIT	_IO('m', 3)	/* don't hold */
#define MUNGEIOCTL_SET		_IOW('m', 4, struct addrpair) /* set monitored pair */
#define MUNGEIOCTL_UNSET	_IO('m', 5) /* unset monitored pair */
#define MUNGEIOCTL_STAT		_IOW('m', 6, struct lin_mungestats)
#define MUNGEIOCTL_DEBUG	_IOW('m', 7, int)
/* also FIOASYNC on/off */

/* Kernel stuff */
#ifdef __KERNEL__
/* Hash table for hitbox entries */
struct lin_hitbox {
	struct lin_hitstats stats;
#define hit_entry	stats.hitreq
	struct lin_hitbox *next;
};

struct hit_packetinfo {
	struct sk_buff *skb;
	struct device *dev;
	struct lin_hitbox *hitme;
	unsigned long flags;
};

typedef int (*packet_munger)(struct sk_buff *, struct device *,
	struct packet_type *, struct lin_hitbox *);

extern void addmunge(packet_munger munger);
extern void rmmunge(packet_munger munger);

struct lin_mungebox {
	struct addrpair munge_who;
	unsigned int	munge_flags;		/* only wait-related right now */
#define MUNGE_WAIT	0x1			/* packet should wait */
#define MUNGE_UPDATED	0x2			/* new data available */
	struct wait_queue *reader_wait;
	struct wait_queue *packet_wait;
	struct lin_mungestats munge_stats;
	struct lin_mungebox *next;
};
#endif

#endif /* _NISTNET_H */
